#include <cstdlib>
#include <GL/glut.h>
#include <vector>
#include <fstream>
#include "Core_Mesh/Mesh.h"
#include "Core_Mesh/Iterators.h"
#include <iostream>
#include <hash_map>
#include "Renderer.h"
#include "Core_Mesh/Loop.h"
#include <queue>

using namespace XMeshLib;

Renderer theRenderer;
Mesh * cmesh;
std::vector<Loop> boundaryLoops;

void renderScene(void);
void mouseMove(int x, int y){theRenderer.MouseMove(x,y);}
void mouseClick(int button , int state, int x, int y){theRenderer.MouseClick(button, state, x, y);}
void reshapeScene(GLsizei width, GLsizei height){theRenderer.ReshapeScene(width, height);}
void keyBoard(unsigned char key, int x, int y){theRenderer.KeyBoard(key,x,y);} 

void HW2();

int main(int argc, char **argv) {
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH );
	glutInitWindowPosition(theRenderer.win_x,theRenderer.win_y);
	glutInitWindowSize(theRenderer.win_width, theRenderer.win_height);
	glutCreateWindow("Homework 2 Solution");
	glutDisplayFunc(renderScene);
	glutReshapeFunc(reshapeScene);
	glutMouseFunc(mouseClick);
	glutMotionFunc(mouseMove);
	glutKeyboardFunc(keyBoard);

	cmesh = new Mesh();
	bool flag = cmesh->read(argv[1]);
	if (!flag)
	{
		std::cerr << "Fail to read " << argv[1] << std::endl;
		return 0;
	}
	theRenderer.SetMesh(cmesh);
	theRenderer.InitGL();

	HW2();
	
	glutMainLoop();
	delete cmesh;
	return 1;
}

void HW2()
{
	//Euler Number
	int x = cmesh->numFaces() - cmesh->numEdges() + cmesh->numVertices();
	std::cout << "Euler # = " << x << std::endl;
	
	//Boundary Number
	stdext::hash_map<Halfedge *, bool> bHEVisited;
	Halfedge * boundHE1 = NULL;
	for (MeshEdgeIterator eit(cmesh); !eit.end(); ++eit)
	{
		Edge * e = *eit;
		if (e->boundary())
		{
			boundHE1 = e->he(0);
			bHEVisited[boundHE1]=false;			
		}
	}
		
	while(boundHE1)
	{
		Loop bLoop;
		bLoop.TraceBoundaryLoop(boundHE1);
		boundaryLoops.push_back(bLoop);		
		//Mark them
		for (int i=0;i<bLoop.lenth();++i)
		{
			bHEVisited[bLoop.he(i)]=true;
		}
		boundHE1=NULL;
		for (stdext::hash_map<Halfedge *, bool>::iterator heit=bHEVisited.begin(); 
			heit!=bHEVisited.end(); ++heit)
		{
			Halfedge * cbhe = heit->first;
			if (bHEVisited[cbhe]==false)
			{
				boundHE1 = cbhe;
				break;
			}
		}
	}
	int b = boundaryLoops.size();		
	std::cout << "Boundary # = " << b << std::endl;

	//Connected Component Number
	std::queue<Face *> flist;
	stdext::hash_map<Face *, bool> fVisited;
	Face * startF;
	for (MeshFaceIterator fit(cmesh); !fit.end();++fit)
	{
		Face * f = *fit;
		fVisited[f]=false;
		startF = f;
	}
	int c= 0;
	flist.push(startF);
	fVisited[startF]=true;
	while(!flist.empty())
	{
		Face * cf = flist.front();
		flist.pop();
		Halfedge * he = cf->he();
		std::vector<Face *> nfs;
		nfs.clear();
		for (int i=0;i<3;++i)
		{
			Halfedge * the = he->he_twin();
			if (the)
				nfs.push_back(the->face());
			he = he->he_next();
		}
		for (int i=0;i<nfs.size();++i)
		{
			Face * nf = nfs[i];
			if (!fVisited[nf])
			{
				fVisited[nf]=true;
				flist.push(nf);
			}
		}
		if (flist.empty())
		{
			c++;
			for (stdext::hash_map<Face *, bool>::iterator vfit=fVisited.begin();
				vfit!=fVisited.end();++vfit)
			{
				Face * f = vfit->first;
				if (!fVisited[f])
				{
					flist.push(f);
					fVisited[f]=true;
					break;
				}
			}
		}
	}
	
	std::cout << "Connected Component # = " << c << std::endl;


}


void renderScene(void) 
{
	theRenderer.RenderAll(boundaryLoops);
	//theRenderer.RenderMesh();
	//for (int i=0;i<boundaryLoops.size();++i)
	//{
	//	theRenderer.RenderEdges(boundaryLoops[i].helist);
	//}
}
